@@ -1,129 +1,129 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-#![cfg_attr(feature = "unstable-simd", feature(core_intrinsics))]
-#![cfg_attr(feature = "unstable-simd", feature(optimize_attribute))]
+#![cfg_attr(feature = "intrinsics", feature(core_intrinsics))]
+#![cfg_attr(feature = "optimize", feature(optimize_attribute))]
 #![allow(unused_unsafe)]
+#![allow(clippy::explicit_auto_deref)]
 #![allow(clippy::missing_safety_doc)]
 #![allow(clippy::redundant_field_names)]
+#![allow(clippy::unnecessary_unwrap)]
 #![allow(clippy::upper_case_acronyms)]
 #![allow(clippy::zero_prefixed_literal)]
+#![allow(non_camel_case_types)]
 
 #[macro_use]
 mod util;
 
 mod deserialize;
-mod exc;
+mod error;
 mod ffi;
 mod opt;
 mod serialize;
 mod typeref;
 mod unicode;
 
+#[cfg(feature = "yyjson")]
+mod yyjson;
+
 use pyo3_ffi::*;
 use std::borrow::Cow;
 use std::os::raw::c_char;
+use std::os::raw::c_int;
+use std::os::raw::c_void;
 
 #[allow(unused_imports)]
-use core::ptr::{null, null_mut, NonNull};
+use std::ptr::{null, null_mut, NonNull};
 
-const DUMPS_DOC: &str =
-    "dumps(obj, /, default=None, option=None)\n--\n\nSerialize Python objects to JSON.\0";
-const LOADS_DOC: &str = "loads(obj, /)\n--\n\nDeserialize JSON to Python objects.\0";
+#[cfg(Py_3_10)]
+macro_rules! add {
+    ($mptr:expr, $name:expr, $obj:expr) => {
+        PyModule_AddObjectRef($mptr, $name.as_ptr() as *const c_char, $obj);
+    };
+}
+
+#[cfg(not(Py_3_10))]
+macro_rules! add {
+    ($mptr:expr, $name:expr, $obj:expr) => {
+        PyModule_AddObject($mptr, $name.as_ptr() as *const c_char, $obj);
+    };
+}
 
 macro_rules! opt {
     ($mptr:expr, $name:expr, $opt:expr) => {
-        unsafe {
-            #[cfg(all(not(target_os = "windows"), target_pointer_width = "64"))]
-            PyModule_AddIntConstant($mptr, $name.as_ptr() as *const c_char, $opt as i64);
-            #[cfg(all(not(target_os = "windows"), target_pointer_width = "32"))]
-            PyModule_AddIntConstant($mptr, $name.as_ptr() as *const c_char, $opt as i32);
-            #[cfg(target_os = "windows")]
-            PyModule_AddIntConstant($mptr, $name.as_ptr() as *const c_char, $opt as i32);
-        }
+        #[cfg(all(not(target_os = "windows"), target_pointer_width = "64"))]
+        PyModule_AddIntConstant($mptr, $name.as_ptr() as *const c_char, $opt as i64);
+        #[cfg(all(not(target_os = "windows"), target_pointer_width = "32"))]
+        PyModule_AddIntConstant($mptr, $name.as_ptr() as *const c_char, $opt as i32);
+        #[cfg(target_os = "windows")]
+        PyModule_AddIntConstant($mptr, $name.as_ptr() as *const c_char, $opt as i32);
     };
 }
 
 #[allow(non_snake_case)]
 #[no_mangle]
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
-pub unsafe extern "C" fn PyInit_orjson() -> *mut PyObject {
-    let init = PyModuleDef {
-        m_base: PyModuleDef_HEAD_INIT,
-        m_name: "orjson\0".as_ptr() as *const c_char,
-        m_doc: null(),
-        m_size: 0,
-        m_methods: null_mut(),
-        m_slots: null_mut(),
-        m_traverse: None,
-        m_clear: None,
-        m_free: None,
-    };
-    let mptr = PyModule_Create(Box::into_raw(Box::new(init)));
-
-    let version = env!("CARGO_PKG_VERSION");
-    unsafe {
-        PyModule_AddObject(
-            mptr,
-            "__version__\0".as_ptr() as *const c_char,
-            PyUnicode_FromStringAndSize(version.as_ptr() as *const c_char, version.len() as isize),
-        )
-    };
-
-    let wrapped_dumps: PyMethodDef;
-
-    #[cfg(Py_3_8)]
+#[cfg_attr(feature = "optimize", optimize(size))]
+pub unsafe extern "C" fn orjson_init_exec(mptr: *mut PyObject) -> c_int {
+    typeref::init_typerefs();
     {
-        wrapped_dumps = PyMethodDef {
-            ml_name: "dumps\0".as_ptr() as *const c_char,
-            ml_meth: PyMethodDefPointer {
-                _PyCFunctionFastWithKeywords: dumps,
-            },
-            ml_flags: pyo3_ffi::METH_FASTCALL | METH_KEYWORDS,
-            ml_doc: DUMPS_DOC.as_ptr() as *const c_char,
-        };
+        let version = env!("CARGO_PKG_VERSION");
+        let pyversion =
+            PyUnicode_FromStringAndSize(version.as_ptr() as *const c_char, version.len() as isize);
+        add!(mptr, "__version__\0", pyversion);
     }
-    #[cfg(not(Py_3_8))]
     {
-        wrapped_dumps = PyMethodDef {
-            ml_name: "dumps\0".as_ptr() as *const c_char,
-            ml_meth: PyMethodDefPointer {
-                PyCFunctionWithKeywords: dumps,
-            },
-            ml_flags: METH_VARARGS | METH_KEYWORDS,
-            ml_doc: DUMPS_DOC.as_ptr() as *const c_char,
-        };
+        let dumps_doc =
+            "dumps(obj, /, default=None, option=None)\n--\n\nSerialize Python objects to JSON.\0";
+
+        let wrapped_dumps: PyMethodDef;
+
+        #[cfg(Py_3_8)]
+        {
+            wrapped_dumps = PyMethodDef {
+                ml_name: "dumps\0".as_ptr() as *const c_char,
+                ml_meth: PyMethodDefPointer {
+                    _PyCFunctionFastWithKeywords: dumps,
+                },
+                ml_flags: pyo3_ffi::METH_FASTCALL | METH_KEYWORDS,
+                ml_doc: dumps_doc.as_ptr() as *const c_char,
+            };
+        }
+        #[cfg(not(Py_3_8))]
+        {
+            wrapped_dumps = PyMethodDef {
+                ml_name: "dumps\0".as_ptr() as *const c_char,
+                ml_meth: PyMethodDefPointer {
+                    PyCFunctionWithKeywords: dumps,
+                },
+                ml_flags: METH_VARARGS | METH_KEYWORDS,
+                ml_doc: dumps_doc.as_ptr() as *const c_char,
+            };
+        }
+
+        let func = PyCFunction_NewEx(
+            Box::into_raw(Box::new(wrapped_dumps)),
+            null_mut(),
+            PyUnicode_InternFromString("orjson\0".as_ptr() as *const c_char),
+        );
+        add!(mptr, "dumps\0", func);
     }
-    unsafe {
-        PyModule_AddObject(
-            mptr,
-            "dumps\0".as_ptr() as *const c_char,
-            PyCFunction_NewEx(
-                Box::into_raw(Box::new(wrapped_dumps)),
-                null_mut(),
-                PyUnicode_InternFromString("orjson\0".as_ptr() as *const c_char),
-            ),
-        )
-    };
 
-    let wrapped_loads = PyMethodDef {
-        ml_name: "loads\0".as_ptr() as *const c_char,
-        ml_meth: PyMethodDefPointer { PyCFunction: loads },
-        ml_flags: METH_O,
-        ml_doc: LOADS_DOC.as_ptr() as *const c_char,
-    };
+    {
+        let loads_doc = "loads(obj, /)\n--\n\nDeserialize JSON to Python objects.\0";
 
-    unsafe {
-        PyModule_AddObject(
-            mptr,
-            "loads\0".as_ptr() as *const c_char,
-            PyCFunction_NewEx(
-                Box::into_raw(Box::new(wrapped_loads)),
-                null_mut(),
-                PyUnicode_InternFromString("orjson\0".as_ptr() as *const c_char),
-            ),
-        )
-    };
+        let wrapped_loads = PyMethodDef {
+            ml_name: "loads\0".as_ptr() as *const c_char,
+            ml_meth: PyMethodDefPointer { PyCFunction: loads },
+            ml_flags: METH_O,
+            ml_doc: loads_doc.as_ptr() as *const c_char,
+        };
+        let func = PyCFunction_NewEx(
+            Box::into_raw(Box::new(wrapped_loads)),
+            null_mut(),
+            PyUnicode_InternFromString("orjson\0".as_ptr() as *const c_char),
+        );
+        add!(mptr, "loads\0", func);
+    }
 
     opt!(mptr, "OPT_APPEND_NEWLINE\0", opt::APPEND_NEWLINE);
     opt!(mptr, "OPT_INDENT_2\0", opt::INDENT_2);
@@ -152,20 +152,8 @@ pub unsafe extern "C" fn PyInit_orjson() -> *mut PyObject {
     opt!(mptr, "OPT_STRICT_INTEGER\0", opt::STRICT_INTEGER);
     opt!(mptr, "OPT_UTC_Z\0", opt::UTC_Z);
 
-    typeref::init_typerefs();
-
-    unsafe {
-        PyModule_AddObject(
-            mptr,
-            "JSONDecodeError\0".as_ptr() as *const c_char,
-            typeref::JsonDecodeError,
-        );
-        PyModule_AddObject(
-            mptr,
-            "JSONEncodeError\0".as_ptr() as *const c_char,
-            typeref::JsonEncodeError,
-        )
-    };
+    add!(mptr, "JSONDecodeError\0", typeref::JsonDecodeError);
+    add!(mptr, "JSONEncodeError\0", typeref::JsonEncodeError);
 
     // maturin>=0.11.0 creates a python package that imports *, hiding dunder by default
     let all: [&str; 20] = [
@@ -200,25 +188,61 @@ pub unsafe extern "C" fn PyInit_orjson() -> *mut PyObject {
         )
     }
 
-    unsafe {
-        PyModule_AddObject(mptr, "__all__\0".as_ptr() as *const c_char, pyall);
-    };
+    add!(mptr, "__all__\0", pyall);
+    0
+}
 
-    mptr
+#[allow(non_snake_case)]
+#[no_mangle]
+#[cold]
+#[cfg_attr(feature = "optimize", optimize(size))]
+pub unsafe extern "C" fn PyInit_orjson() -> *mut PyModuleDef {
+    let mod_slots: Box<[PyModuleDef_Slot; 2]> = Box::new([
+        PyModuleDef_Slot {
+            slot: Py_mod_exec,
+            value: orjson_init_exec as *mut c_void,
+        },
+        PyModuleDef_Slot {
+            slot: 0,
+            value: null_mut(),
+        },
+    ]);
+
+    let init = Box::new(PyModuleDef {
+        m_base: PyModuleDef_HEAD_INIT,
+        m_name: "orjson\0".as_ptr() as *const c_char,
+        m_doc: null(),
+        m_size: 0,
+        m_methods: null_mut(),
+        m_slots: Box::into_raw(mod_slots) as *mut PyModuleDef_Slot,
+        m_traverse: None,
+        m_clear: None,
+        m_free: None,
+    });
+    let init_ptr = Box::into_raw(init);
+    PyModuleDef_Init(init_ptr);
+    init_ptr
 }
 
 #[cold]
 #[inline(never)]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 fn raise_loads_exception(err: deserialize::DeserializeError) -> *mut PyObject {
-    let pos = err.pos() as i64;
+    let pos = err.pos();
     let msg = err.message;
-    let doc = err.data;
+    let doc = match err.data {
+        Some(as_str) => unsafe {
+            PyUnicode_FromStringAndSize(as_str.as_ptr() as *const c_char, as_str.len() as isize)
+        },
+        None => {
+            ffi!(Py_INCREF(crate::typeref::EMPTY_UNICODE));
+            unsafe { crate::typeref::EMPTY_UNICODE }
+        }
+    };
     unsafe {
         let err_msg =
             PyUnicode_FromStringAndSize(msg.as_ptr() as *const c_char, msg.len() as isize);
         let args = PyTuple_New(3);
-        let doc = PyUnicode_FromStringAndSize(doc.as_ptr() as *const c_char, doc.len() as isize);
         let pos = PyLong_FromLongLong(pos);
         PyTuple_SET_ITEM(args, 0, err_msg);
         PyTuple_SET_ITEM(args, 1, doc);
@@ -231,7 +255,7 @@ fn raise_loads_exception(err: deserialize::DeserializeError) -> *mut PyObject {
 
 #[cold]
 #[inline(never)]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 fn raise_dumps_exception(msg: Cow<str>) -> *mut PyObject {
     unsafe {
         let err_msg =
@@ -274,7 +298,7 @@ pub unsafe extern "C" fn dumps(
         optsptr = Some(NonNull::new_unchecked(*args.offset(2)));
     }
     if !kwnames.is_null() {
-        for i in 0..=PyTuple_GET_SIZE(kwnames) - 1 {
+        for i in 0..=Py_SIZE(kwnames).saturating_sub(1) {
             let arg = PyTuple_GET_ITEM(kwnames, i as Py_ssize_t);
             if arg == typeref::DEFAULT {
                 if unlikely!(num_args & 2 == 2) {
@@ -300,12 +324,14 @@ pub unsafe extern "C" fn dumps(
 
     let mut optsbits: i32 = 0;
     if let Some(opts) = optsptr {
-        if (*opts.as_ptr()).ob_type != typeref::INT_TYPE {
-            return raise_dumps_exception(Cow::Borrowed("Invalid opts"));
-        }
-        optsbits = PyLong_AsLong(optsptr.unwrap().as_ptr()) as i32;
-        if !(0..=opt::MAX_OPT).contains(&optsbits) {
+        if opts.as_ptr() == typeref::NONE {
+        } else if (*opts.as_ptr()).ob_type != typeref::INT_TYPE {
             return raise_dumps_exception(Cow::Borrowed("Invalid opts"));
+        } else {
+            optsbits = PyLong_AsLong(optsptr.unwrap().as_ptr()) as i32;
+            if !(0..=opt::MAX_OPT).contains(&optsbits) {
+                return raise_dumps_exception(Cow::Borrowed("Invalid opts"));
+            }
         }
     }
 
@@ -327,7 +353,7 @@ pub unsafe extern "C" fn dumps(
 
     let obj = PyTuple_GET_ITEM(args, 0);
 
-    let num_args = PyTuple_GET_SIZE(args);
+    let num_args = Py_SIZE(args);
     if unlikely!(num_args == 0) {
         return raise_dumps_exception(Cow::Borrowed(
             "dumps() missing 1 required positional argument: 'obj'",
@@ -341,12 +367,7 @@ pub unsafe extern "C" fn dumps(
     }
 
     if !kwds.is_null() {
-        let len = unsafe { crate::ffi::PyDict_GET_SIZE(kwds) };
-        let mut pos = 0isize;
-        let mut arg: *mut PyObject = null_mut();
-        let mut val: *mut PyObject = null_mut();
-        for _ in 0..=len.saturating_sub(1) {
-            unsafe { _PyDict_Next(kwds, &mut pos, &mut arg, &mut val, null_mut()) };
+        for (arg, val) in crate::ffi::PyDictIter::from_pyobject(kwds) {
             if arg == typeref::DEFAULT {
                 if unlikely!(num_args & 2 == 2) {
                     return raise_dumps_exception(Cow::Borrowed(
@@ -373,12 +394,14 @@ pub unsafe extern "C" fn dumps(
 
     let mut optsbits: i32 = 0;
     if let Some(opts) = optsptr {
-        if (*opts.as_ptr()).ob_type != typeref::INT_TYPE {
-            return raise_dumps_exception(Cow::Borrowed("Invalid opts"));
-        }
-        optsbits = PyLong_AsLong(optsptr.unwrap().as_ptr()) as i32;
-        if optsbits < 0 || optsbits > opt::MAX_OPT {
+        if opts.as_ptr() == typeref::NONE {
+        } else if (*opts.as_ptr()).ob_type != typeref::INT_TYPE {
             return raise_dumps_exception(Cow::Borrowed("Invalid opts"));
+        } else {
+            optsbits = PyLong_AsLong(optsptr.unwrap().as_ptr()) as i32;
+            if !(0..=opt::MAX_OPT).contains(&optsbits) {
+                return raise_dumps_exception(Cow::Borrowed("Invalid opts"));
+            }
         }
     }
 
