@@ -1,8 +1,8 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use crate::exc::*;
 use crate::opt::*;
 use crate::serialize::datetimelike::{DateTimeBuffer, DateTimeError, DateTimeLike, Offset};
+use crate::serialize::error::*;
 use crate::typeref::*;
 use serde::ser::{Serialize, Serializer};
 
@@ -39,7 +39,7 @@ impl Date {
     }
     pub fn write_buf(&self, buf: &mut DateTimeBuffer) {
         {
-            let year = ffi!(PyDateTime_GET_YEAR(self.ptr)) as i32;
+            let year = ffi!(PyDateTime_GET_YEAR(self.ptr));
             let mut yearbuf = itoa::Buffer::new();
             let formatted = yearbuf.format(year);
             if unlikely!(year < 1000) {
@@ -60,7 +60,7 @@ impl Date {
         }
     }
 }
-impl<'p> Serialize for Date {
+impl Serialize for Date {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -82,16 +82,16 @@ pub struct Time {
 }
 
 impl Time {
-    pub fn new(ptr: *mut pyo3_ffi::PyObject, opts: Opt) -> Result<Self, TimeError> {
-        if unsafe { (*(ptr as *mut pyo3_ffi::PyDateTime_Time)).hastzinfo == 1 } {
-            return Err(TimeError::HasTimezone);
-        }
-        Ok(Time {
+    pub fn new(ptr: *mut pyo3_ffi::PyObject, opts: Opt) -> Self {
+        Time {
             ptr: ptr,
             opts: opts,
-        })
+        }
     }
-    pub fn write_buf(&self, buf: &mut DateTimeBuffer) {
+    pub fn write_buf(&self, buf: &mut DateTimeBuffer) -> Result<(), TimeError> {
+        if unsafe { (*(self.ptr as *mut pyo3_ffi::PyDateTime_Time)).hastzinfo == 1 } {
+            return Err(TimeError::HasTimezone);
+        }
         let hour = ffi!(PyDateTime_TIME_GET_HOUR(self.ptr)) as u8;
         write_double_digit!(buf, hour);
         buf.push(b':');
@@ -104,17 +104,20 @@ impl Time {
             let microsecond = ffi!(PyDateTime_TIME_GET_MICROSECOND(self.ptr)) as u32;
             write_microsecond!(buf, microsecond);
         }
+        Ok(())
     }
 }
 
-impl<'p> Serialize for Time {
+impl Serialize for Time {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
         let mut buf = DateTimeBuffer::new();
-        self.write_buf(&mut buf);
+        if self.write_buf(&mut buf).is_err() {
+            err!(SerializeError::DatetimeLibraryUnsupported)
+        };
         serializer.serialize_str(str_from_slice!(buf.as_ptr(), buf.len()))
     }
 }
@@ -168,7 +171,7 @@ impl DateTimeLike for DateTime {
             // pendulum
             let py_offset = call_method!(self.ptr, UTCOFFSET_METHOD_STR);
             let offset = Offset {
-                second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)) as i32,
+                second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)),
                 day: ffi!(PyDateTime_DELTA_GET_DAYS(py_offset)),
             };
             ffi!(Py_DECREF(py_offset));
@@ -179,7 +182,7 @@ impl DateTimeLike for DateTime {
             let py_offset = call_method!(method_ptr, UTCOFFSET_METHOD_STR);
             ffi!(Py_DECREF(method_ptr));
             let offset = Offset {
-                second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)) as i32,
+                second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)),
                 day: ffi!(PyDateTime_DELTA_GET_DAYS(py_offset)),
             };
             ffi!(Py_DECREF(py_offset));
@@ -188,7 +191,7 @@ impl DateTimeLike for DateTime {
             // dateutil/arrow, datetime.timezone.utc
             let py_offset = call_method!(tzinfo, UTCOFFSET_METHOD_STR, self.ptr);
             let offset = Offset {
-                second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)) as i32,
+                second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)),
                 day: ffi!(PyDateTime_DELTA_GET_DAYS(py_offset)),
             };
             ffi!(Py_DECREF(py_offset));
@@ -208,7 +211,7 @@ impl DateTimeLike for DateTime {
                 // zoneinfo
                 let py_offset = call_method!(tzinfo, UTCOFFSET_METHOD_STR, self.ptr);
                 let offset = Offset {
-                    second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)) as i32,
+                    second: ffi!(PyDateTime_DELTA_GET_SECONDS(py_offset)),
                     day: ffi!(PyDateTime_DELTA_GET_DAYS(py_offset)),
                 };
                 ffi!(Py_DECREF(py_offset));
@@ -229,7 +232,7 @@ impl DateTimeLike for DateTime {
     }
 }
 
-impl<'p> Serialize for DateTime {
+impl Serialize for DateTime {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
