@@ -1,19 +1,18 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use crate::exc::*;
-use crate::ffi::PyDict_GET_SIZE;
+use crate::ffi::PyDictIter;
 use crate::opt::*;
 use crate::serialize::datetime::*;
 use crate::serialize::datetimelike::*;
+use crate::serialize::error::*;
 use crate::serialize::serializer::pyobject_to_obtype;
 use crate::serialize::serializer::*;
 use crate::serialize::uuid::*;
 use crate::typeref::*;
 use crate::unicode::*;
-use inlinable_string::InlinableString;
+use compact_str::CompactString;
 use serde::ser::{Serialize, SerializeMap, Serializer};
 use smallvec::SmallVec;
-use std::ptr::addr_of_mut;
 use std::ptr::NonNull;
 
 pub struct Dict {
@@ -36,52 +35,36 @@ impl Dict {
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
-            recursion: recursion,
+            recursion: recursion + 1,
             default: default,
         }
     }
 }
 
-impl<'p> Serialize for Dict {
+impl Serialize for Dict {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
         let mut map = serializer.serialize_map(None).unwrap();
-        let mut pos = 0isize;
-        let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-        let mut key: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-        let mut value: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-        for _ in 0..=unsafe { PyDict_GET_SIZE(self.ptr) as usize } - 1 {
-            unsafe {
-                pyo3_ffi::_PyDict_Next(
-                    self.ptr,
-                    addr_of_mut!(pos),
-                    addr_of_mut!(key),
-                    addr_of_mut!(value),
-                    std::ptr::null_mut(),
-                )
-            };
-            let value = PyObjectSerializer::new(
-                value,
-                self.opts,
-                self.default_calls,
-                self.recursion + 1,
-                self.default,
-            );
+        for (key, value) in PyDictIter::from_pyobject(self.ptr) {
             if unlikely!(unsafe { ob_type!(key) != STR_TYPE }) {
                 err!(SerializeError::KeyMustBeStr)
             }
-            {
-                let data = read_utf8_from_str(key, &mut str_size);
-                if unlikely!(data.is_null()) {
-                    err!(SerializeError::InvalidStr)
-                }
-                map.serialize_key(str_from_slice!(data, str_size)).unwrap();
+            let key_as_str = unicode_to_str(key);
+            if unlikely!(key_as_str.is_none()) {
+                err!(SerializeError::InvalidStr)
             }
-
-            map.serialize_value(&value)?;
+            let pyvalue = PyObjectSerializer::new(
+                value,
+                self.opts,
+                self.default_calls,
+                self.recursion,
+                self.default,
+            );
+            map.serialize_key(key_as_str.unwrap()).unwrap();
+            map.serialize_value(&pyvalue)?;
         }
         map.end()
     }
@@ -107,59 +90,45 @@ impl DictSortedKey {
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
-            recursion: recursion,
+            recursion: recursion + 1,
             default: default,
         }
     }
 }
 
-impl<'p> Serialize for DictSortedKey {
+impl Serialize for DictSortedKey {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let len = unsafe { PyDict_GET_SIZE(self.ptr) as usize };
+        let len = ffi!(Py_SIZE(self.ptr)) as usize;
         let mut items: SmallVec<[(&str, *mut pyo3_ffi::PyObject); 8]> =
             SmallVec::with_capacity(len);
-        let mut pos = 0isize;
-        let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-        let mut key: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-        let mut value: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-        for _ in 0..=len - 1 {
-            unsafe {
-                pyo3_ffi::_PyDict_Next(
-                    self.ptr,
-                    addr_of_mut!(pos),
-                    addr_of_mut!(key),
-                    addr_of_mut!(value),
-                    std::ptr::null_mut(),
-                )
-            };
+        for (key, value) in PyDictIter::from_pyobject(self.ptr) {
             if unlikely!(unsafe { ob_type!(key) != STR_TYPE }) {
                 err!(SerializeError::KeyMustBeStr)
             }
-            let data = read_utf8_from_str(key, &mut str_size);
-            if unlikely!(data.is_null()) {
+            let data = unicode_to_str(key);
+            if unlikely!(data.is_none()) {
                 err!(SerializeError::InvalidStr)
             }
-            items.push((str_from_slice!(data, str_size), value));
+            items.push((data.unwrap(), value));
         }
 
         items.sort_unstable_by(|a, b| a.0.cmp(b.0));
 
         let mut map = serializer.serialize_map(None).unwrap();
         for (key, val) in items.iter() {
-            map.serialize_entry(
-                key,
-                &PyObjectSerializer::new(
-                    *val,
-                    self.opts,
-                    self.default_calls,
-                    self.recursion + 1,
-                    self.default,
-                ),
-            )?;
+            let pyvalue = PyObjectSerializer::new(
+                *val,
+                self.opts,
+                self.default_calls,
+                self.recursion,
+                self.default,
+            );
+            map.serialize_key(key).unwrap();
+            map.serialize_value(&pyvalue)?;
         }
         map.end()
     }
@@ -185,26 +154,24 @@ impl DictNonStrKey {
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
-            recursion: recursion,
+            recursion: recursion + 1,
             default: default,
         }
     }
 
     fn pyobject_to_string(
-        &self,
         key: *mut pyo3_ffi::PyObject,
         opts: crate::opt::Opt,
-    ) -> Result<InlinableString, SerializeError> {
+    ) -> Result<CompactString, SerializeError> {
         match pyobject_to_obtype(key, opts) {
-            ObType::None => Ok(InlinableString::from("null")),
+            ObType::None => Ok(CompactString::from("null")),
             ObType::Bool => {
-                let key_as_str: &str;
-                if unsafe { key == TRUE } {
-                    key_as_str = "true";
+                let key_as_str = if unsafe { key == TRUE } {
+                    "true"
                 } else {
-                    key_as_str = "false";
-                }
-                Ok(InlinableString::from(key_as_str))
+                    "false"
+                };
+                Ok(CompactString::from(key_as_str))
             }
             ObType::Int => {
                 let ival = ffi!(PyLong_AsLongLong(key));
@@ -214,17 +181,17 @@ impl DictNonStrKey {
                     if unlikely!(uval == u64::MAX && !ffi!(PyErr_Occurred()).is_null()) {
                         return Err(SerializeError::DictIntegerKey64Bit);
                     }
-                    Ok(InlinableString::from(itoa::Buffer::new().format(uval)))
+                    Ok(CompactString::from(itoa::Buffer::new().format(uval)))
                 } else {
-                    Ok(InlinableString::from(itoa::Buffer::new().format(ival)))
+                    Ok(CompactString::from(itoa::Buffer::new().format(ival)))
                 }
             }
             ObType::Float => {
                 let val = ffi!(PyFloat_AS_DOUBLE(key));
                 if !val.is_finite() {
-                    Ok(InlinableString::from("null"))
+                    Ok(CompactString::from("null"))
                 } else {
-                    Ok(InlinableString::from(ryu::Buffer::new().format_finite(val)))
+                    Ok(CompactString::from(ryu::Buffer::new().format_finite(val)))
                 }
             }
             ObType::Datetime => {
@@ -234,51 +201,49 @@ impl DictNonStrKey {
                     return Err(SerializeError::DatetimeLibraryUnsupported);
                 }
                 let key_as_str = str_from_slice!(buf.as_ptr(), buf.len());
-                Ok(InlinableString::from(key_as_str))
+                Ok(CompactString::from(key_as_str))
             }
             ObType::Date => {
                 let mut buf = DateTimeBuffer::new();
                 Date::new(key).write_buf(&mut buf);
                 let key_as_str = str_from_slice!(buf.as_ptr(), buf.len());
-                Ok(InlinableString::from(key_as_str))
+                Ok(CompactString::from(key_as_str))
             }
-            ObType::Time => match Time::new(key, opts) {
-                Ok(val) => {
-                    let mut buf = DateTimeBuffer::new();
-                    val.write_buf(&mut buf);
-                    let key_as_str = str_from_slice!(buf.as_ptr(), buf.len());
-                    Ok(InlinableString::from(key_as_str))
+            ObType::Time => {
+                let mut buf = DateTimeBuffer::new();
+                let time = Time::new(key, opts);
+                if time.write_buf(&mut buf).is_err() {
+                    return Err(SerializeError::TimeHasTzinfo);
                 }
-                Err(TimeError::HasTimezone) => Err(SerializeError::TimeHasTzinfo),
-            },
+                let key_as_str = str_from_slice!(buf.as_ptr(), buf.len());
+                Ok(CompactString::from(key_as_str))
+            }
             ObType::Uuid => {
                 let mut buf = arrayvec::ArrayVec::<u8, 36>::new();
                 UUID::new(key).write_buf(&mut buf);
                 let key_as_str = str_from_slice!(buf.as_ptr(), buf.len());
-                Ok(InlinableString::from(key_as_str))
+                Ok(CompactString::from(key_as_str))
             }
             ObType::Enum => {
                 let value = ffi!(PyObject_GetAttr(key, VALUE_STR));
                 ffi!(Py_DECREF(value));
-                self.pyobject_to_string(value, opts)
+                Self::pyobject_to_string(value, opts)
             }
             ObType::Str => {
                 // because of ObType::Enum
-                let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-                let uni = read_utf8_from_str(key, &mut str_size);
-                if unlikely!(uni.is_null()) {
+                let uni = unicode_to_str(key);
+                if unlikely!(uni.is_none()) {
                     Err(SerializeError::InvalidStr)
                 } else {
-                    Ok(InlinableString::from(str_from_slice!(uni, str_size)))
+                    Ok(CompactString::from(uni.unwrap()))
                 }
             }
             ObType::StrSubclass => {
-                let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-                let uni = ffi!(PyUnicode_AsUTF8AndSize(key, &mut str_size)) as *const u8;
-                if unlikely!(uni.is_null()) {
+                let uni = unicode_to_str_via_ffi(key);
+                if unlikely!(uni.is_none()) {
                     Err(SerializeError::InvalidStr)
                 } else {
-                    Ok(InlinableString::from(str_from_slice!(uni, str_size)))
+                    Ok(CompactString::from(uni.unwrap()))
                 }
             }
             ObType::Tuple
@@ -292,41 +257,25 @@ impl DictNonStrKey {
     }
 }
 
-impl<'p> Serialize for DictNonStrKey {
+impl Serialize for DictNonStrKey {
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let len = unsafe { PyDict_GET_SIZE(self.ptr) as usize };
-        let mut items: SmallVec<[(InlinableString, *mut pyo3_ffi::PyObject); 8]> =
+        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let mut items: SmallVec<[(CompactString, *mut pyo3_ffi::PyObject); 8]> =
             SmallVec::with_capacity(len);
-        let mut pos = 0isize;
-        let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-        let mut key: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-        let mut value: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
         let opts = self.opts & NOT_PASSTHROUGH;
-        for _ in 0..=len - 1 {
-            unsafe {
-                pyo3_ffi::_PyDict_Next(
-                    self.ptr,
-                    addr_of_mut!(pos),
-                    addr_of_mut!(key),
-                    addr_of_mut!(value),
-                    std::ptr::null_mut(),
-                )
-            };
+        for (key, value) in PyDictIter::from_pyobject(self.ptr) {
             if is_type!(ob_type!(key), STR_TYPE) {
-                let data = read_utf8_from_str(key, &mut str_size);
-                if unlikely!(data.is_null()) {
+                let uni = unicode_to_str(key);
+                if unlikely!(uni.is_none()) {
                     err!(SerializeError::InvalidStr)
                 }
-                items.push((
-                    InlinableString::from(str_from_slice!(data, str_size)),
-                    value,
-                ));
+                items.push((CompactString::from(uni.unwrap()), value));
             } else {
-                match self.pyobject_to_string(key, opts) {
+                match Self::pyobject_to_string(key, opts) {
                     Ok(key_as_str) => items.push((key_as_str, value)),
                     Err(err) => err!(err),
                 }
@@ -339,16 +288,15 @@ impl<'p> Serialize for DictNonStrKey {
 
         let mut map = serializer.serialize_map(None).unwrap();
         for (key, val) in items.iter() {
-            map.serialize_entry(
-                str_from_slice!(key.as_ptr(), key.len()),
-                &PyObjectSerializer::new(
-                    *val,
-                    self.opts,
-                    self.default_calls,
-                    self.recursion + 1,
-                    self.default,
-                ),
-            )?;
+            let pyvalue = PyObjectSerializer::new(
+                *val,
+                self.opts,
+                self.default_calls,
+                self.recursion,
+                self.default,
+            );
+            map.serialize_key(key).unwrap();
+            map.serialize_value(&pyvalue)?;
         }
         map.end()
     }
