@@ -26,34 +26,34 @@ impl ListSerializer {
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
-            recursion: recursion,
+            recursion: recursion + 1,
             default: default,
         }
     }
 }
 
-impl<'p> Serialize for ListSerializer {
+impl Serialize for ListSerializer {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut seq = serializer.serialize_seq(None).unwrap();
-        let slice: &[*mut pyo3_ffi::PyObject] = unsafe {
-            std::slice::from_raw_parts(
-                (*(self.ptr as *mut pyo3_ffi::PyListObject)).ob_item,
-                ffi!(PyList_GET_SIZE(self.ptr)) as usize,
-            )
-        };
-        for &each in slice {
-            let value = PyObjectSerializer::new(
-                each,
-                self.opts,
-                self.default_calls,
-                self.recursion + 1,
-                self.default,
-            );
-            seq.serialize_element(&value)?;
+        if ffi!(Py_SIZE(self.ptr)) == 0 {
+            serializer.serialize_seq(Some(0)).unwrap().end()
+        } else {
+            let mut seq = serializer.serialize_seq(None).unwrap();
+            for i in 0..=ffi!(Py_SIZE(self.ptr)) as usize - 1 {
+                let elem =
+                    unsafe { *((*(self.ptr as *mut pyo3_ffi::PyListObject)).ob_item).add(i) };
+                let value = PyObjectSerializer::new(
+                    elem,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                );
+                seq.serialize_element(&value)?;
+            }
+            seq.end()
         }
-        seq.end()
     }
 }
