@@ -6,9 +6,11 @@ mod datetime;
 mod datetimelike;
 mod default;
 mod dict;
+mod error;
 mod int;
 mod list;
 mod numpy;
+mod pyenum;
 mod serializer;
 mod str;
 mod tuple;
