@@ -1,5 +1,7 @@
 use crate::opt::*;
 use crate::serialize::datetimelike::{DateTimeBuffer, DateTimeError, DateTimeLike, Offset};
+use crate::serialize::default::*;
+use crate::serialize::error::*;
 use crate::typeref::{ARRAY_STRUCT_STR, DESCR_STR, DTYPE_STR, NUMPY_TYPES};
 use chrono::{Datelike, NaiveDate, NaiveDateTime, Timelike};
 use pyo3_ffi::*;
@@ -8,6 +10,65 @@ use std::convert::TryInto;
 use std::fmt;
 use std::ops::DerefMut;
 use std::os::raw::{c_char, c_int, c_void};
+use std::ptr::NonNull;
+
+pub struct NumpySerializer {
+    ptr: *mut pyo3_ffi::PyObject,
+    opts: Opt,
+    default_calls: u8,
+    recursion: u8,
+    default: Option<NonNull<pyo3_ffi::PyObject>>,
+}
+
+impl NumpySerializer {
+    pub fn new(
+        ptr: *mut pyo3_ffi::PyObject,
+        opts: Opt,
+        default_calls: u8,
+        recursion: u8,
+        default: Option<NonNull<pyo3_ffi::PyObject>>,
+    ) -> Self {
+        NumpySerializer {
+            ptr: ptr,
+            opts: opts,
+            default_calls: default_calls,
+            recursion: recursion,
+            default: default,
+        }
+    }
+}
+
+impl Serialize for NumpySerializer {
+    #[cold]
+    #[inline(never)]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        match NumpyArray::new(self.ptr, self.opts) {
+            Ok(val) => val.serialize(serializer),
+            Err(PyArrayError::Malformed) => err!(SerializeError::NumpyMalformed),
+            Err(PyArrayError::NotContiguous) | Err(PyArrayError::UnsupportedDataType)
+                if self.default.is_some() =>
+            {
+                DefaultSerializer::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                )
+                .serialize(serializer)
+            }
+            Err(PyArrayError::NotContiguous) => {
+                err!(SerializeError::NumpyNotCContiguous)
+            }
+            Err(PyArrayError::UnsupportedDataType) => {
+                err!(SerializeError::NumpyUnsupportedDatatype)
+            }
+        }
+    }
+}
 
 macro_rules! slice {
     ($ptr:expr, $size:expr) => {
@@ -23,11 +84,13 @@ pub fn is_numpy_scalar(ob_type: *mut PyTypeObject) -> bool {
         ob_type == scalar_types.float64
             || ob_type == scalar_types.float32
             || ob_type == scalar_types.int64
+            || ob_type == scalar_types.int16
             || ob_type == scalar_types.int32
             || ob_type == scalar_types.int8
             || ob_type == scalar_types.uint64
             || ob_type == scalar_types.uint32
             || ob_type == scalar_types.uint8
+            || ob_type == scalar_types.uint16
             || ob_type == scalar_types.bool_
             || ob_type == scalar_types.datetime64
     }
@@ -73,9 +136,11 @@ pub enum ItemType {
     F32,
     F64,
     I8,
+    I16,
     I32,
     I64,
     U8,
+    U16,
     U32,
     U64,
 }
@@ -91,9 +156,11 @@ impl ItemType {
             (102, 4) => Some(ItemType::F32),
             (102, 8) => Some(ItemType::F64),
             (105, 1) => Some(ItemType::I8),
+            (105, 2) => Some(ItemType::I16),
             (105, 4) => Some(ItemType::I32),
             (105, 8) => Some(ItemType::I64),
             (117, 1) => Some(ItemType::U8),
+            (117, 2) => Some(ItemType::U16),
             (117, 4) => Some(ItemType::U32),
             (117, 8) => Some(ItemType::U64),
             _ => None,
@@ -123,7 +190,8 @@ pub struct NumpyArray {
     opts: Opt,
 }
 
-impl<'a> NumpyArray {
+impl NumpyArray {
+    #[cold]
     #[inline(never)]
     pub fn new(ptr: *mut PyObject, opts: Opt) -> Result<Self, PyArrayError> {
         let capsule = ffi!(PyObject_GetAttr(ptr, ARRAY_STRUCT_STR));
@@ -161,7 +229,7 @@ impl<'a> NumpyArray {
         }
     }
 
-    fn from_parent(&self, position: Vec<isize>, num_children: usize) -> Self {
+    fn child_from_parent(&self, position: Vec<isize>, num_children: usize) -> Self {
         let mut arr = NumpyArray {
             array: self.array,
             position: position,
@@ -178,15 +246,15 @@ impl<'a> NumpyArray {
     fn build(&mut self) {
         if self.depth < self.dimensions() - 1 {
             for i in 0..=self.shape()[self.depth] - 1 {
-                let mut position: Vec<isize> = self.position.iter().copied().collect();
+                let mut position: Vec<isize> = self.position.to_vec();
                 position[self.depth] = i;
-                let num_children: usize;
-                if self.depth < self.dimensions() - 2 {
-                    num_children = self.shape()[self.depth + 1] as usize;
+                let num_children: usize = if self.depth < self.dimensions() - 2 {
+                    self.shape()[self.depth + 1] as usize
                 } else {
-                    num_children = 0;
-                }
-                self.children.push(self.from_parent(position, num_children))
+                    0
+                };
+                self.children
+                    .push(self.child_from_parent(position, num_children))
             }
         }
     }
@@ -228,88 +296,135 @@ impl Drop for NumpyArray {
     }
 }
 
-impl<'p> Serialize for NumpyArray {
+impl Serialize for NumpyArray {
+    #[cold]
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut seq = serializer.serialize_seq(None).unwrap();
-
-        if self.depth >= self.shape().len() || self.shape()[self.depth] != 0 {
-            if !self.children.is_empty() {
-                for child in &self.children {
-                    seq.serialize_element(child).unwrap();
-                }
-            } else {
-                let data_ptr = self.data();
-                let num_items = self.num_items();
-                match self.kind {
-                    ItemType::F64 => {
-                        let slice: &[f64] = slice!(data_ptr as *const f64, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeF64 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::F32 => {
-                        let slice: &[f32] = slice!(data_ptr as *const f32, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeF32 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::I64 => {
-                        let slice: &[i64] = slice!(data_ptr as *const i64, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeI64 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::I32 => {
-                        let slice: &[i32] = slice!(data_ptr as *const i32, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeI32 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::I8 => {
-                        let slice: &[i8] = slice!(data_ptr as *const i8, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeI8 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::U8 => {
-                        let slice: &[u8] = slice!(data_ptr as *const u8, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeU8 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::U32 => {
-                        let slice: &[u32] = slice!(data_ptr as *const u32, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeU32 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::U64 => {
-                        let slice: &[u64] = slice!(data_ptr as *const u64, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeU64 { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::BOOL => {
-                        let slice: &[u8] = slice!(data_ptr as *const u8, num_items);
-                        for &each in slice.iter() {
-                            seq.serialize_element(&DataTypeBOOL { obj: each }).unwrap();
-                        }
-                    }
-                    ItemType::DATETIME64(unit) => {
-                        let slice: &[i64] = slice!(data_ptr as *const i64, num_items);
-                        for &each in slice.iter() {
-                            let dt = unit
-                                .datetime(each, self.opts)
-                                .map_err(NumpyDateTimeError::into_serde_err)?;
-                            seq.serialize_element(&dt).unwrap();
-                        }
-                    }
-                }
+        if unlikely!(!(self.depth >= self.dimensions() || self.shape()[self.depth] != 0)) {
+            serializer.serialize_seq(Some(0)).unwrap().end()
+        } else if !self.children.is_empty() {
+            let mut seq = serializer.serialize_seq(None).unwrap();
+            for child in &self.children {
+                seq.serialize_element(child).unwrap();
             }
+            seq.end()
+        } else {
+            match self.kind {
+                ItemType::F64 => {
+                    NumpyF64Array::new(slice!(self.data() as *const f64, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::F32 => {
+                    NumpyF32Array::new(slice!(self.data() as *const f32, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::U64 => {
+                    NumpyU64Array::new(slice!(self.data() as *const u64, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::U32 => {
+                    NumpyU32Array::new(slice!(self.data() as *const u32, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::U16 => {
+                    NumpyU16Array::new(slice!(self.data() as *const u16, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::U8 => {
+                    NumpyU8Array::new(slice!(self.data() as *const u8, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::I64 => {
+                    NumpyI64Array::new(slice!(self.data() as *const i64, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::I32 => {
+                    NumpyI32Array::new(slice!(self.data() as *const i32, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::I16 => {
+                    NumpyI16Array::new(slice!(self.data() as *const i16, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::I8 => {
+                    NumpyI8Array::new(slice!(self.data() as *const i8, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::BOOL => {
+                    NumpyBoolArray::new(slice!(self.data() as *const u8, self.num_items()))
+                        .serialize(serializer)
+                }
+                ItemType::DATETIME64(unit) => NumpyDatetime64Array::new(
+                    slice!(self.data() as *const i64, self.num_items()),
+                    unit,
+                    self.opts,
+                )
+                .serialize(serializer),
+            }
+        }
+    }
+}
+
+#[repr(transparent)]
+struct NumpyF64Array<'a> {
+    data: &'a [f64],
+}
+
+impl<'a> NumpyF64Array<'a> {
+    fn new(data: &'a [f64]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyF64Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeF64 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeF64 {
+    obj: f64,
+}
+
+impl Serialize for DataTypeF64 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_f64(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyF32Array<'a> {
+    data: &'a [f32],
+}
+
+impl<'a> NumpyF32Array<'a> {
+    fn new(data: &'a [f32]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyF32Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeF32 { obj: each }).unwrap();
         }
         seq.end()
     }
@@ -320,7 +435,7 @@ struct DataTypeF32 {
     obj: f32,
 }
 
-impl<'p> Serialize for DataTypeF32 {
+impl Serialize for DataTypeF32 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -330,86 +445,26 @@ impl<'p> Serialize for DataTypeF32 {
 }
 
 #[repr(transparent)]
-pub struct DataTypeF64 {
-    obj: f64,
+struct NumpyU64Array<'a> {
+    data: &'a [u64],
 }
 
-impl<'p> Serialize for DataTypeF64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_f64(self.obj)
+impl<'a> NumpyU64Array<'a> {
+    fn new(data: &'a [u64]) -> Self {
+        Self { data }
     }
 }
 
-#[repr(transparent)]
-pub struct DataTypeI8 {
-    obj: i8,
-}
-
-impl<'p> Serialize for DataTypeI8 {
+impl<'a> Serialize for NumpyU64Array<'a> {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        serializer.serialize_i8(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI32 {
-    obj: i32,
-}
-
-impl<'p> Serialize for DataTypeI32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i32(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI64 {
-    obj: i64,
-}
-
-impl<'p> Serialize for DataTypeI64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i64(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU8 {
-    obj: u8,
-}
-
-impl<'p> Serialize for DataTypeU8 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u8(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU32 {
-    obj: u32,
-}
-
-impl<'p> Serialize for DataTypeU32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u32(self.obj)
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeU64 { obj: each }).unwrap();
+        }
+        seq.end()
     }
 }
 
@@ -418,7 +473,7 @@ pub struct DataTypeU64 {
     obj: u64,
 }
 
-impl<'p> Serialize for DataTypeU64 {
+impl Serialize for DataTypeU64 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -428,11 +483,301 @@ impl<'p> Serialize for DataTypeU64 {
 }
 
 #[repr(transparent)]
-pub struct DataTypeBOOL {
+struct NumpyU32Array<'a> {
+    data: &'a [u32],
+}
+
+impl<'a> NumpyU32Array<'a> {
+    fn new(data: &'a [u32]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyU32Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeU32 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeU32 {
+    obj: u32,
+}
+
+impl Serialize for DataTypeU32 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u32(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyU16Array<'a> {
+    data: &'a [u16],
+}
+
+impl<'a> NumpyU16Array<'a> {
+    fn new(data: &'a [u16]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyU16Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeU16 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeU16 {
+    obj: u16,
+}
+
+impl Serialize for DataTypeU16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u16(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyI64Array<'a> {
+    data: &'a [i64],
+}
+
+impl<'a> NumpyI64Array<'a> {
+    fn new(data: &'a [i64]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyI64Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeI64 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI64 {
+    obj: i64,
+}
+
+impl Serialize for DataTypeI64 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i64(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyI32Array<'a> {
+    data: &'a [i32],
+}
+
+impl<'a> NumpyI32Array<'a> {
+    fn new(data: &'a [i32]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyI32Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeI32 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI32 {
+    obj: i32,
+}
+
+impl Serialize for DataTypeI32 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i32(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyI16Array<'a> {
+    data: &'a [i16],
+}
+
+impl<'a> NumpyI16Array<'a> {
+    fn new(data: &'a [i16]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyI16Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeI16 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI16 {
+    obj: i16,
+}
+
+impl Serialize for DataTypeI16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i16(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyI8Array<'a> {
+    data: &'a [i8],
+}
+
+impl<'a> NumpyI8Array<'a> {
+    fn new(data: &'a [i8]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyI8Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeI8 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI8 {
+    obj: i8,
+}
+
+impl Serialize for DataTypeI8 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i8(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyU8Array<'a> {
+    data: &'a [u8],
+}
+
+impl<'a> NumpyU8Array<'a> {
+    fn new(data: &'a [u8]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyU8Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeU8 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeU8 {
     obj: u8,
 }
 
-impl<'p> Serialize for DataTypeBOOL {
+impl Serialize for DataTypeU8 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u8(self.obj)
+    }
+}
+
+#[repr(transparent)]
+struct NumpyBoolArray<'a> {
+    data: &'a [u8],
+}
+
+impl<'a> NumpyBoolArray<'a> {
+    fn new(data: &'a [u8]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyBoolArray<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeBool { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeBool {
+    obj: u8,
+}
+
+impl Serialize for DataTypeBool {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -452,7 +797,8 @@ impl NumpyScalar {
     }
 }
 
-impl<'p> Serialize for NumpyScalar {
+impl Serialize for NumpyScalar {
+    #[cold]
     #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -469,12 +815,16 @@ impl<'p> Serialize for NumpyScalar {
                 (*(self.ptr as *mut NumpyInt64)).serialize(serializer)
             } else if ob_type == scalar_types.int32 {
                 (*(self.ptr as *mut NumpyInt32)).serialize(serializer)
+            } else if ob_type == scalar_types.int16 {
+                (*(self.ptr as *mut NumpyInt16)).serialize(serializer)
             } else if ob_type == scalar_types.int8 {
                 (*(self.ptr as *mut NumpyInt8)).serialize(serializer)
             } else if ob_type == scalar_types.uint64 {
                 (*(self.ptr as *mut NumpyUint64)).serialize(serializer)
             } else if ob_type == scalar_types.uint32 {
                 (*(self.ptr as *mut NumpyUint32)).serialize(serializer)
+            } else if ob_type == scalar_types.uint16 {
+                (*(self.ptr as *mut NumpyUint16)).serialize(serializer)
             } else if ob_type == scalar_types.uint8 {
                 (*(self.ptr as *mut NumpyUint8)).serialize(serializer)
             } else if ob_type == scalar_types.bool_ {
@@ -500,7 +850,7 @@ pub struct NumpyInt8 {
     value: i8,
 }
 
-impl<'p> Serialize for NumpyInt8 {
+impl Serialize for NumpyInt8 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -509,6 +859,22 @@ impl<'p> Serialize for NumpyInt8 {
     }
 }
 
+#[repr(C)]
+pub struct NumpyInt16 {
+    pub ob_refcnt: Py_ssize_t,
+    pub ob_type: *mut PyTypeObject,
+    pub value: i16,
+}
+
+impl Serialize for NumpyInt16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i16(self.value)
+    }
+}
+
 #[repr(C)]
 pub struct NumpyInt32 {
     ob_refcnt: Py_ssize_t,
@@ -516,7 +882,7 @@ pub struct NumpyInt32 {
     value: i32,
 }
 
-impl<'p> Serialize for NumpyInt32 {
+impl Serialize for NumpyInt32 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -532,7 +898,7 @@ pub struct NumpyInt64 {
     value: i64,
 }
 
-impl<'p> Serialize for NumpyInt64 {
+impl Serialize for NumpyInt64 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -548,7 +914,7 @@ pub struct NumpyUint8 {
     value: u8,
 }
 
-impl<'p> Serialize for NumpyUint8 {
+impl Serialize for NumpyUint8 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -557,6 +923,22 @@ impl<'p> Serialize for NumpyUint8 {
     }
 }
 
+#[repr(C)]
+pub struct NumpyUint16 {
+    pub ob_refcnt: Py_ssize_t,
+    pub ob_type: *mut PyTypeObject,
+    pub value: u16,
+}
+
+impl Serialize for NumpyUint16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u16(self.value)
+    }
+}
+
 #[repr(C)]
 pub struct NumpyUint32 {
     ob_refcnt: Py_ssize_t,
@@ -564,7 +946,7 @@ pub struct NumpyUint32 {
     value: u32,
 }
 
-impl<'p> Serialize for NumpyUint32 {
+impl Serialize for NumpyUint32 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -580,7 +962,7 @@ pub struct NumpyUint64 {
     value: u64,
 }
 
-impl<'p> Serialize for NumpyUint64 {
+impl Serialize for NumpyUint64 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -596,7 +978,7 @@ pub struct NumpyFloat32 {
     value: f32,
 }
 
-impl<'p> Serialize for NumpyFloat32 {
+impl Serialize for NumpyFloat32 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -612,7 +994,7 @@ pub struct NumpyFloat64 {
     value: f64,
 }
 
-impl<'p> Serialize for NumpyFloat64 {
+impl Serialize for NumpyFloat64 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -628,7 +1010,7 @@ pub struct NumpyBool {
     value: bool,
 }
 
-impl<'p> Serialize for NumpyBool {
+impl Serialize for NumpyBool {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -641,7 +1023,7 @@ impl<'p> Serialize for NumpyBool {
 ///
 /// See
 /// https://github.com/numpy/numpy/blob/fc8e3bbe419748ac5c6b7f3d0845e4bafa74644b/numpy/core/include/numpy/ndarraytypes.h#L268-L282.
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy)]
 pub enum NumpyDatetimeUnit {
     NaT,
     Years,
@@ -662,7 +1044,7 @@ pub enum NumpyDatetimeUnit {
 
 impl fmt::Display for NumpyDatetimeUnit {
     #[cold]
-    #[cfg_attr(feature = "unstable-simd", optimize(size))]
+    #[cfg_attr(feature = "optimize", optimize(size))]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let unit = match self {
             Self::NaT => "NaT",
@@ -685,7 +1067,7 @@ impl fmt::Display for NumpyDatetimeUnit {
     }
 }
 
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy)]
 enum NumpyDateTimeError {
     UnsupportedUnit(NumpyDatetimeUnit),
     Unrepresentable { unit: NumpyDatetimeUnit, val: i64 },
@@ -693,7 +1075,7 @@ enum NumpyDateTimeError {
 
 impl NumpyDateTimeError {
     #[cold]
-    #[cfg_attr(feature = "unstable-simd", optimize(size))]
+    #[cfg_attr(feature = "optimize", optimize(size))]
     fn into_serde_err<T: ser::Error>(self) -> T {
         let err = match self {
             Self::UnsupportedUnit(unit) => format!("unsupported numpy.datetime64 unit: {}", unit),
@@ -722,15 +1104,13 @@ impl NumpyDatetimeUnit {
         let el0 = ffi!(PyList_GET_ITEM(descr, 0));
         ffi!(Py_DECREF(descr));
         let descr_str = ffi!(PyTuple_GET_ITEM(el0, 1));
-        let mut str_size: pyo3_ffi::Py_ssize_t = 0;
-        let uni = crate::unicode::read_utf8_from_str(descr_str, &mut str_size);
-        if str_size < 5 {
+        let uni = crate::unicode::unicode_to_str(descr_str).unwrap();
+        if uni.len() < 5 {
             return Self::NaT;
         }
-        let fmt = str_from_slice!(uni, str_size);
         // unit descriptions are found at
         // https://github.com/numpy/numpy/blob/b235f9e701e14ed6f6f6dcba885f7986a833743f/numpy/core/src/multiarray/datetime.c#L79-L96.
-        match &fmt[4..fmt.len() - 1] {
+        match &uni[4..uni.len() - 1] {
             "Y" => Self::Years,
             "M" => Self::Months,
             "W" => Self::Weeks,
@@ -754,15 +1134,17 @@ impl NumpyDatetimeUnit {
     /// Returns an `Err(NumpyDateTimeError)` if the value is invalid for this unit.
     fn datetime(&self, val: i64, opts: Opt) -> Result<NumpyDatetime64Repr, NumpyDateTimeError> {
         match self {
-            Self::Years => Ok(NaiveDate::from_ymd(
+            Self::Years => Ok(NaiveDate::from_ymd_opt(
                 (val + 1970)
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
                 1,
                 1,
             )
-            .and_hms(0, 0, 0)),
-            Self::Months => Ok(NaiveDate::from_ymd(
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap()),
+            Self::Months => Ok(NaiveDate::from_ymd_opt(
                 (val / 12 + 1970)
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
@@ -771,36 +1153,72 @@ impl NumpyDatetimeUnit {
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
                 1,
             )
-            .and_hms(0, 0, 0)),
-            Self::Weeks => Ok(NaiveDateTime::from_timestamp(val * 7 * 24 * 60 * 60, 0)),
-            Self::Days => Ok(NaiveDateTime::from_timestamp(val * 24 * 60 * 60, 0)),
-            Self::Hours => Ok(NaiveDateTime::from_timestamp(val * 60 * 60, 0)),
-            Self::Minutes => Ok(NaiveDateTime::from_timestamp(val * 60, 0)),
-            Self::Seconds => Ok(NaiveDateTime::from_timestamp(val, 0)),
-            Self::Milliseconds => Ok(NaiveDateTime::from_timestamp(
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap()),
+            Self::Weeks => {
+                Ok(NaiveDateTime::from_timestamp_opt(val * 7 * 24 * 60 * 60, 0).unwrap())
+            }
+            Self::Days => Ok(NaiveDateTime::from_timestamp_opt(val * 24 * 60 * 60, 0).unwrap()),
+            Self::Hours => Ok(NaiveDateTime::from_timestamp_opt(val * 60 * 60, 0).unwrap()),
+            Self::Minutes => Ok(NaiveDateTime::from_timestamp_opt(val * 60, 0).unwrap()),
+            Self::Seconds => Ok(NaiveDateTime::from_timestamp_opt(val, 0).unwrap()),
+            Self::Milliseconds => Ok(NaiveDateTime::from_timestamp_opt(
                 val / 1_000,
                 (val % 1_000 * 1_000_000)
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
-            )),
-            Self::Microseconds => Ok(NaiveDateTime::from_timestamp(
+            )
+            .unwrap()),
+            Self::Microseconds => Ok(NaiveDateTime::from_timestamp_opt(
                 val / 1_000_000,
                 (val % 1_000_000 * 1_000)
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
-            )),
-            Self::Nanoseconds => Ok(NaiveDateTime::from_timestamp(
+            )
+            .unwrap()),
+            Self::Nanoseconds => Ok(NaiveDateTime::from_timestamp_opt(
                 val / 1_000_000_000,
                 (val % 1_000_000_000)
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
-            )),
+            )
+            .unwrap()),
             _ => Err(NumpyDateTimeError::UnsupportedUnit(*self)),
         }
         .map(|dt| NumpyDatetime64Repr { dt, opts })
     }
 }
 
+struct NumpyDatetime64Array<'a> {
+    data: &'a [i64],
+    unit: NumpyDatetimeUnit,
+    opts: Opt,
+}
+
+impl<'a> NumpyDatetime64Array<'a> {
+    fn new(data: &'a [i64], unit: NumpyDatetimeUnit, opts: Opt) -> Self {
+        Self { data, unit, opts }
+    }
+}
+
+impl<'a> Serialize for NumpyDatetime64Array<'a> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            let dt = self
+                .unit
+                .datetime(each, self.opts)
+                .map_err(NumpyDateTimeError::into_serde_err)?;
+            seq.serialize_element(&dt).unwrap();
+        }
+        seq.end()
+    }
+}
+
 #[repr(C)]
 pub struct NumpyDatetime64 {
     ob_refcnt: Py_ssize_t,
@@ -857,7 +1275,7 @@ impl Serialize for NumpyDatetime64Repr {
         S: Serializer,
     {
         let mut buf = DateTimeBuffer::new();
-        self.write_buf(&mut buf, self.opts).unwrap();
+        let _ = self.write_buf(&mut buf, self.opts);
         serializer.collect_str(str_from_slice!(buf.as_ptr(), buf.len()))
     }
 }
