@@ -1,21 +1,22 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use crate::exc::*;
-use crate::ffi::{PyDict_GET_SIZE, PyTypeObject};
+use crate::ffi::PyTypeObject;
 use crate::opt::*;
 use crate::serialize::dataclass::*;
 use crate::serialize::datetime::*;
 use crate::serialize::default::*;
 use crate::serialize::dict::*;
+use crate::serialize::error::*;
 use crate::serialize::int::*;
 use crate::serialize::list::*;
 use crate::serialize::numpy::*;
+use crate::serialize::pyenum::EnumSerializer;
 use crate::serialize::str::*;
 use crate::serialize::tuple::*;
 use crate::serialize::uuid::*;
 use crate::serialize::writer::*;
 use crate::typeref::*;
-use serde::ser::{Serialize, SerializeMap, SerializeSeq, Serializer};
+use serde::ser::{Serialize, SerializeMap, Serializer};
 use std::io::Write;
 use std::ptr::NonNull;
 
@@ -26,14 +27,13 @@ pub fn serialize(
     default: Option<NonNull<pyo3_ffi::PyObject>>,
     opts: Opt,
 ) -> Result<NonNull<pyo3_ffi::PyObject>, String> {
-    let mut buf = BytesWriter::new();
+    let mut buf = BytesWriter::default();
     let obj = PyObjectSerializer::new(ptr, opts, 0, 0, default);
-    let res;
-    if opts & INDENT_2 != INDENT_2 {
-        res = serde_json::to_writer(&mut buf, &obj);
+    let res = if opts & INDENT_2 != INDENT_2 {
+        serde_json::to_writer(&mut buf, &obj)
     } else {
-        res = serde_json::to_writer_pretty(&mut buf, &obj);
-    }
+        serde_json::to_writer_pretty(&mut buf, &obj)
+    };
     match res {
         Ok(_) => {
             if opts & APPEND_NEWLINE != 0 {
@@ -48,6 +48,7 @@ pub fn serialize(
     }
 }
 
+#[repr(u32)]
 #[derive(Copy, Clone)]
 pub enum ObType {
     Str,
@@ -97,12 +98,12 @@ pub fn pyobject_to_obtype(obj: *mut pyo3_ffi::PyObject, opts: Opt) -> ObType {
 
 macro_rules! is_subclass {
     ($ob_type:expr, $flag:ident) => {
-        unsafe { (((*$ob_type).tp_flags & pyo3_ffi::$flag) != 0) }
+        (((*$ob_type).tp_flags & pyo3_ffi::$flag) != 0)
     };
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 #[inline(never)]
 pub fn pyobject_to_obtype_unlikely(obj: *mut pyo3_ffi::PyObject, opts: Opt) -> ObType {
     unsafe {
@@ -149,7 +150,6 @@ pub fn pyobject_to_obtype_unlikely(obj: *mut pyo3_ffi::PyObject, opts: Opt) -> O
 
 pub struct PyObjectSerializer {
     ptr: *mut pyo3_ffi::PyObject,
-    obtype: ObType,
     opts: Opt,
     default_calls: u8,
     recursion: u8,
@@ -166,7 +166,6 @@ impl PyObjectSerializer {
     ) -> Self {
         PyObjectSerializer {
             ptr: ptr,
-            obtype: pyobject_to_obtype(ptr, opts),
             opts: opts,
             default_calls: default_calls,
             recursion: recursion,
@@ -175,12 +174,12 @@ impl PyObjectSerializer {
     }
 }
 
-impl<'p> Serialize for PyObjectSerializer {
+impl Serialize for PyObjectSerializer {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        match self.obtype {
+        match pyobject_to_obtype(self.ptr, self.opts) {
             ObType::Str => StrSerializer::new(self.ptr).serialize(serializer),
             ObType::StrSubclass => StrSubclassSerializer::new(self.ptr).serialize(serializer),
             ObType::Int => {
@@ -195,16 +194,13 @@ impl<'p> Serialize for PyObjectSerializer {
             ObType::Bool => serializer.serialize_bool(unsafe { self.ptr == TRUE }),
             ObType::Datetime => DateTime::new(self.ptr, self.opts).serialize(serializer),
             ObType::Date => Date::new(self.ptr).serialize(serializer),
-            ObType::Time => match Time::new(self.ptr, self.opts) {
-                Ok(val) => val.serialize(serializer),
-                Err(TimeError::HasTimezone) => err!(SerializeError::TimeHasTzinfo),
-            },
+            ObType::Time => Time::new(self.ptr, self.opts).serialize(serializer),
             ObType::Uuid => UUID::new(self.ptr).serialize(serializer),
             ObType::Dict => {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(SerializeError::RecursionLimit)
                 }
-                if unlikely!(unsafe { PyDict_GET_SIZE(self.ptr) } == 0) {
+                if ffi!(Py_SIZE(self.ptr)) == 0 {
                     serializer.serialize_map(Some(0)).unwrap().end()
                 } else if self.opts & SORT_OR_NON_STR_KEYS == 0 {
                     Dict::new(
@@ -239,18 +235,14 @@ impl<'p> Serialize for PyObjectSerializer {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(SerializeError::RecursionLimit)
                 }
-                if unlikely!(ffi!(PyList_GET_SIZE(self.ptr)) == 0) {
-                    serializer.serialize_seq(Some(0)).unwrap().end()
-                } else {
-                    ListSerializer::new(
-                        self.ptr,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
-                }
+                ListSerializer::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                )
+                .serialize(serializer)
             }
             ObType::Tuple => TupleSerializer::new(
                 self.ptr,
@@ -269,7 +261,7 @@ impl<'p> Serialize for PyObjectSerializer {
                 if unlikely!(
                     dict.is_null() || ffi!(PyDict_Contains((*ob_type).tp_dict, SLOTS_STR)) == 1
                 ) {
-                    unsafe { pyo3_ffi::PyErr_Clear() };
+                    ffi!(PyErr_Clear());
                     DataclassFallbackSerializer::new(
                         self.ptr,
                         self.opts,
@@ -290,40 +282,22 @@ impl<'p> Serialize for PyObjectSerializer {
                     .serialize(serializer)
                 }
             }
-            ObType::Enum => {
-                let value = ffi!(PyObject_GetAttr(self.ptr, VALUE_STR));
-                ffi!(Py_DECREF(value));
-                PyObjectSerializer::new(
-                    value,
-                    self.opts,
-                    self.default_calls,
-                    self.recursion,
-                    self.default,
-                )
-                .serialize(serializer)
-            }
-            ObType::NumpyArray => match NumpyArray::new(self.ptr, self.opts) {
-                Ok(val) => val.serialize(serializer),
-                Err(PyArrayError::Malformed) => err!(SerializeError::NumpyMalformed),
-                Err(PyArrayError::NotContiguous) | Err(PyArrayError::UnsupportedDataType)
-                    if self.default.is_some() =>
-                {
-                    DefaultSerializer::new(
-                        self.ptr,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
-                }
-                Err(PyArrayError::NotContiguous) => {
-                    err!(SerializeError::NumpyNotCContiguous)
-                }
-                Err(PyArrayError::UnsupportedDataType) => {
-                    err!(SerializeError::NumpyUnsupportedDatatype)
-                }
-            },
+            ObType::Enum => EnumSerializer::new(
+                self.ptr,
+                self.opts,
+                self.default_calls,
+                self.recursion,
+                self.default,
+            )
+            .serialize(serializer),
+            ObType::NumpyArray => NumpySerializer::new(
+                self.ptr,
+                self.opts,
+                self.default_calls,
+                self.recursion,
+                self.default,
+            )
+            .serialize(serializer),
             ObType::NumpyScalar => NumpyScalar::new(self.ptr, self.opts).serialize(serializer),
             ObType::Unknown => DefaultSerializer::new(
                 self.ptr,
