@@ -13,9 +13,11 @@ pub struct NumpyTypes {
     pub float32: *mut PyTypeObject,
     pub int64: *mut PyTypeObject,
     pub int32: *mut PyTypeObject,
+    pub int16: *mut PyTypeObject,
     pub int8: *mut PyTypeObject,
     pub uint64: *mut PyTypeObject,
     pub uint32: *mut PyTypeObject,
+    pub uint16: *mut PyTypeObject,
     pub uint8: *mut PyTypeObject,
     pub bool_: *mut PyTypeObject,
     pub datetime64: *mut PyTypeObject,
@@ -76,6 +78,28 @@ pub static mut HASH_BUILDER: Lazy<ahash::RandomState> = Lazy::new(|| unsafe {
         BYTES_TYPE as u64,
     )
 });
+#[cfg(feature = "yyjson")]
+pub const YYJSON_BUFFER_SIZE: usize = 1024 * 1024 * 8;
+
+#[cfg(feature = "yyjson")]
+pub static mut YYJSON_ALLOC: Lazy<crate::yyjson::yyjson_alc> = Lazy::new(|| unsafe {
+    let buffer = std::alloc::alloc(std::alloc::Layout::from_size_align_unchecked(
+        YYJSON_BUFFER_SIZE,
+        std::mem::align_of::<*mut ()>(),
+    ));
+    let mut alloc = crate::yyjson::yyjson_alc {
+        malloc: None,
+        realloc: None,
+        free: None,
+        ctx: std::ptr::null_mut(),
+    };
+    crate::yyjson::yyjson_alc_pool_init(
+        &mut alloc,
+        buffer as *mut std::os::raw::c_void,
+        YYJSON_BUFFER_SIZE,
+    );
+    alloc
+});
 
 #[allow(non_upper_case_globals)]
 pub static mut JsonEncodeError: *mut PyObject = 0 as *mut PyObject;
@@ -85,7 +109,7 @@ pub static mut JsonDecodeError: *mut PyObject = 0 as *mut PyObject;
 static INIT: Once = Once::new();
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 pub fn init_typerefs() {
     INIT.call_once(|| unsafe {
         assert!(crate::deserialize::KEY_MAP
@@ -146,12 +170,13 @@ pub fn init_typerefs() {
         DEFAULT = PyUnicode_InternFromString("default\0".as_ptr() as *const c_char);
         OPTION = PyUnicode_InternFromString("option\0".as_ptr() as *const c_char);
         JsonEncodeError = pyo3_ffi::PyExc_TypeError;
+        Py_INCREF(JsonEncodeError);
         JsonDecodeError = look_up_json_exc();
     });
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_json_exc() -> *mut PyObject {
     let module = PyImport_ImportModule("json\0".as_ptr() as *const c_char);
     let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
@@ -165,11 +190,12 @@ unsafe fn look_up_json_exc() -> *mut PyObject {
     Py_DECREF(ptr);
     Py_DECREF(module_dict);
     Py_DECREF(module);
+    Py_INCREF(res);
     res
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_numpy_type(numpy_module: *mut PyObject, np_type: &str) -> *mut PyTypeObject {
     let mod_dict = PyObject_GenericGetDict(numpy_module, std::ptr::null_mut());
     let ptr = PyMapping_GetItemString(mod_dict, np_type.as_ptr() as *const c_char);
@@ -179,7 +205,7 @@ unsafe fn look_up_numpy_type(numpy_module: *mut PyObject, np_type: &str) -> *mut
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn load_numpy_types() -> Option<NumpyTypes> {
     let numpy = PyImport_ImportModule("numpy\0".as_ptr() as *const c_char);
     if numpy.is_null() {
@@ -192,8 +218,10 @@ unsafe fn load_numpy_types() -> Option<NumpyTypes> {
         float32: look_up_numpy_type(numpy, "float32\0"),
         float64: look_up_numpy_type(numpy, "float64\0"),
         int8: look_up_numpy_type(numpy, "int8\0"),
+        int16: look_up_numpy_type(numpy, "int16\0"),
         int32: look_up_numpy_type(numpy, "int32\0"),
         int64: look_up_numpy_type(numpy, "int64\0"),
+        uint16: look_up_numpy_type(numpy, "uint16\0"),
         uint32: look_up_numpy_type(numpy, "uint32\0"),
         uint64: look_up_numpy_type(numpy, "uint64\0"),
         uint8: look_up_numpy_type(numpy, "uint8\0"),
@@ -205,7 +233,7 @@ unsafe fn load_numpy_types() -> Option<NumpyTypes> {
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_field_type() -> NonNull<PyObject> {
     let module = PyImport_ImportModule("dataclasses\0".as_ptr() as *const c_char);
     let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
@@ -217,7 +245,7 @@ unsafe fn look_up_field_type() -> NonNull<PyObject> {
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_enum_type() -> *mut PyTypeObject {
     let module = PyImport_ImportModule("enum\0".as_ptr() as *const c_char);
     let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
@@ -229,7 +257,7 @@ unsafe fn look_up_enum_type() -> *mut PyTypeObject {
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_uuid_type() -> *mut PyTypeObject {
     let uuid_mod = PyImport_ImportModule("uuid\0".as_ptr() as *const c_char);
     let uuid_mod_dict = PyObject_GenericGetDict(uuid_mod, std::ptr::null_mut());
@@ -242,7 +270,7 @@ unsafe fn look_up_uuid_type() -> *mut PyTypeObject {
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_datetime_type() -> *mut PyTypeObject {
     let datetime = ((*PyDateTimeAPI()).DateTime_FromDateAndTime)(
         1970,
@@ -261,7 +289,7 @@ unsafe fn look_up_datetime_type() -> *mut PyTypeObject {
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_date_type() -> *mut PyTypeObject {
     let date = ((*PyDateTimeAPI()).Date_FromDate)(1, 1, 1, (*(PyDateTimeAPI())).DateType);
     let ptr = (*date).ob_type;
@@ -270,7 +298,7 @@ unsafe fn look_up_date_type() -> *mut PyTypeObject {
 }
 
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_time_type() -> *mut PyTypeObject {
     let time = ((*PyDateTimeAPI()).Time_FromTime)(0, 0, 0, 0, NONE, (*(PyDateTimeAPI())).TimeType);
     let ptr = (*time).ob_type;
@@ -280,7 +308,7 @@ unsafe fn look_up_time_type() -> *mut PyTypeObject {
 
 #[cfg(Py_3_9)]
 #[cold]
-#[cfg_attr(feature = "unstable-simd", optimize(size))]
+#[cfg_attr(feature = "optimize", optimize(size))]
 unsafe fn look_up_zoneinfo_type() -> *mut PyTypeObject {
     let module = PyImport_ImportModule("zoneinfo\0".as_ptr() as *const c_char);
     let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
