@@ -1,6 +1,5 @@
 # SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-import unittest
 
 import pytest
 
@@ -17,108 +16,124 @@ def numpy_default(obj):
 
 
 @pytest.mark.skipif(numpy is None, reason="numpy is not installed")
-class NumpyTests(unittest.TestCase):
+class TestNumpy:
     def test_numpy_array_d1_uintp(self):
-        self.assertEqual(
-            orjson.dumps(
-                numpy.array([0, 18446744073709551615], numpy.uintp),
-                option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[0,18446744073709551615]",
-        )
+        low = numpy.iinfo(numpy.uintp).min
+        high = numpy.iinfo(numpy.uintp).max
+        assert orjson.dumps(
+            numpy.array([low, high], numpy.uintp),
+            option=orjson.OPT_SERIALIZE_NUMPY,
+        ) == f"[{low},{high}]".encode("ascii")
 
     def test_numpy_array_d1_intp(self):
-        self.assertEqual(
-            orjson.dumps(
-                numpy.array([-9223372036854775807, 9223372036854775807], numpy.intp),
-                option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[-9223372036854775807,9223372036854775807]",
-        )
+        low = numpy.iinfo(numpy.intp).min
+        high = numpy.iinfo(numpy.intp).max
+        assert orjson.dumps(
+            numpy.array([low, high], numpy.intp),
+            option=orjson.OPT_SERIALIZE_NUMPY,
+        ) == f"[{low},{high}]".encode("ascii")
 
     def test_numpy_array_d1_i64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([-9223372036854775807, 9223372036854775807], numpy.int64),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[-9223372036854775807,9223372036854775807]",
+            )
+            == b"[-9223372036854775807,9223372036854775807]"
         )
 
     def test_numpy_array_d1_u64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([0, 18446744073709551615], numpy.uint64),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[0,18446744073709551615]",
+            )
+            == b"[0,18446744073709551615]"
         )
 
     def test_numpy_array_d1_i8(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([-128, 127], numpy.int8),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[-128,127]",
+            )
+            == b"[-128,127]"
         )
 
     def test_numpy_array_d1_u8(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([0, 255], numpy.uint8),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[0,255]",
+            )
+            == b"[0,255]"
         )
 
     def test_numpy_array_d1_i32(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([-2147483647, 2147483647], numpy.int32),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[-2147483647,2147483647]",
+            )
+            == b"[-2147483647,2147483647]"
+        )
+
+    def test_numpy_array_d1_i16(self):
+        assert (
+            orjson.dumps(
+                numpy.array([-32768, 32767], numpy.int16),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b"[-32768,32767]"
+        )
+
+    def test_numpy_array_d1_u16(self):
+        assert (
+            orjson.dumps(
+                numpy.array([0, 65535], numpy.uint16),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b"[0,65535]"
         )
 
     def test_numpy_array_d1_u32(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([0, 4294967295], numpy.uint32),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[0,4294967295]",
+            )
+            == b"[0,4294967295]"
         )
 
     def test_numpy_array_d1_f32(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([1.0, 3.4028235e38], numpy.float32),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[1.0,3.4028235e38]",
+            )
+            == b"[1.0,3.4028235e38]"
         )
 
     def test_numpy_array_d1_f64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([1.0, 1.7976931348623157e308], numpy.float64),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[1.0,1.7976931348623157e308]",
+            )
+            == b"[1.0,1.7976931348623157e308]"
         )
 
     def test_numpy_array_d1_bool(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([True, False, False, True]),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[true,false,false,true]",
+            )
+            == b"[true,false,false,true]"
         )
 
     def test_numpy_array_d1_datetime64_years(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -133,12 +148,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["0001-01-01T00:00:00","0970-01-01T00:00:00","1920-01-01T00:00:00","1971-01-01T00:00:00","2021-01-01T00:00:00","2022-01-01T00:00:00","2023-01-01T00:00:00","9999-01-01T00:00:00"]',
+            )
+            == b'["0001-01-01T00:00:00","0970-01-01T00:00:00","1920-01-01T00:00:00","1971-01-01T00:00:00","2021-01-01T00:00:00","2022-01-01T00:00:00","2023-01-01T00:00:00","9999-01-01T00:00:00"]'
         )
 
     def test_numpy_array_d1_datetime64_months(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -148,12 +163,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2022-01-01T00:00:00","2023-01-01T00:00:00"]',
+            )
+            == b'["2021-01-01T00:00:00","2022-01-01T00:00:00","2023-01-01T00:00:00"]'
         )
 
     def test_numpy_array_d1_datetime64_days(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -163,12 +178,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2021-01-01T00:00:00","2021-01-01T00:00:00"]',
+            )
+            == b'["2021-01-01T00:00:00","2021-01-01T00:00:00","2021-01-01T00:00:00"]'
         )
 
     def test_numpy_array_d1_datetime64_hours(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -178,12 +193,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2021-01-01T01:00:00","2021-01-01T02:00:00"]',
+            )
+            == b'["2021-01-01T00:00:00","2021-01-01T01:00:00","2021-01-01T02:00:00"]'
         )
 
     def test_numpy_array_d1_datetime64_minutes(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -193,12 +208,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2021-01-01T00:01:00","2021-01-01T00:02:00"]',
+            )
+            == b'["2021-01-01T00:00:00","2021-01-01T00:01:00","2021-01-01T00:02:00"]'
         )
 
     def test_numpy_array_d1_datetime64_seconds(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -208,12 +223,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2021-01-01T00:00:01","2021-01-01T00:00:02"]',
+            )
+            == b'["2021-01-01T00:00:00","2021-01-01T00:00:01","2021-01-01T00:00:02"]'
         )
 
     def test_numpy_array_d1_datetime64_milliseconds(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -223,12 +238,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2021-01-01T00:00:00.172000","2021-01-01T00:00:00.567000"]',
+            )
+            == b'["2021-01-01T00:00:00","2021-01-01T00:00:00.172000","2021-01-01T00:00:00.567000"]'
         )
 
     def test_numpy_array_d1_datetime64_microseconds(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -238,12 +253,12 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2021-01-01T00:00:00.172000","2021-01-01T00:00:00.567891"]',
+            )
+            == b'["2021-01-01T00:00:00","2021-01-01T00:00:00.172000","2021-01-01T00:00:00.567891"]'
         )
 
     def test_numpy_array_d1_datetime64_nanoseconds(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [
@@ -253,8 +268,8 @@ class NumpyTests(unittest.TestCase):
                     ]
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'["2021-01-01T00:00:00","2021-01-01T00:00:00.172000","2021-01-01T00:00:00.567891"]',
+            )
+            == b'["2021-01-01T00:00:00","2021-01-01T00:00:00.172000","2021-01-01T00:00:00.567891"]'
         )
 
     def test_numpy_array_d1_datetime64_picoseconds(self):
@@ -271,87 +286,81 @@ class NumpyTests(unittest.TestCase):
             )
             assert False
         except TypeError as exc:
-            self.assertEqual(
-                str(exc),
-                "unsupported numpy.datetime64 unit: picoseconds",
-            )
+            assert str(exc) == "unsupported numpy.datetime64 unit: picoseconds"
 
     def test_numpy_array_d2_i64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([[1, 2, 3], [4, 5, 6]], numpy.int64),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[[1,2,3],[4,5,6]]",
+            )
+            == b"[[1,2,3],[4,5,6]]"
         )
 
     def test_numpy_array_d2_f64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], numpy.float64),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[[1.0,2.0,3.0],[4.0,5.0,6.0]]",
+            )
+            == b"[[1.0,2.0,3.0],[4.0,5.0,6.0]]"
         )
 
     def test_numpy_array_d3_i8(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]], numpy.int8),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[[[1,2],[3,4]],[[5,6],[7,8]]]",
+            )
+            == b"[[[1,2],[3,4]],[[5,6],[7,8]]]"
         )
 
     def test_numpy_array_d3_u8(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]], numpy.uint8),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[[[1,2],[3,4]],[[5,6],[7,8]]]",
+            )
+            == b"[[[1,2],[3,4]],[[5,6],[7,8]]]"
         )
 
     def test_numpy_array_d3_i32(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]], numpy.int32),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[[[1,2],[3,4]],[[5,6],[7,8]]]",
+            )
+            == b"[[[1,2],[3,4]],[[5,6],[7,8]]]"
         )
 
     def test_numpy_array_d3_i64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array([[[1, 2], [3, 4], [5, 6], [7, 8]]], numpy.int64),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[[[1,2],[3,4],[5,6],[7,8]]]",
+            )
+            == b"[[[1,2],[3,4],[5,6],[7,8]]]"
         )
 
     def test_numpy_array_d3_f64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.array(
                     [[[1.0, 2.0], [3.0, 4.0]], [[5.0, 6.0], [7.0, 8.0]]], numpy.float64
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b"[[[1.0,2.0],[3.0,4.0]],[[5.0,6.0],[7.0,8.0]]]",
+            )
+            == b"[[[1.0,2.0],[3.0,4.0]],[[5.0,6.0],[7.0,8.0]]]"
         )
 
     def test_numpy_array_fortran(self):
         array = numpy.array([[1, 2], [3, 4]], order="F")
         assert array.flags["F_CONTIGUOUS"] == True
-        with self.assertRaises(orjson.JSONEncodeError):
+        with pytest.raises(orjson.JSONEncodeError):
             orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
-        self.assertEqual(
-            orjson.dumps(
-                array, default=numpy_default, option=orjson.OPT_SERIALIZE_NUMPY
-            ),
-            orjson.dumps(array.tolist()),
-        )
+        assert orjson.dumps(
+            array, default=numpy_default, option=orjson.OPT_SERIALIZE_NUMPY
+        ) == orjson.dumps(array.tolist())
 
     def test_numpy_array_non_contiguous_message(self):
         array = numpy.array([[1, 2], [3, 4]], order="F")
@@ -360,120 +369,117 @@ class NumpyTests(unittest.TestCase):
             orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
             assert False
         except TypeError as exc:
-            self.assertEqual(
-                str(exc),
-                "numpy array is not C contiguous; use ndarray.tolist() in default",
+            assert (
+                str(exc)
+                == "numpy array is not C contiguous; use ndarray.tolist() in default"
             )
 
     def test_numpy_array_unsupported_dtype(self):
         array = numpy.array([[1, 2], [3, 4]], numpy.float16)
-        with self.assertRaises(orjson.JSONEncodeError) as cm:
+        with pytest.raises(orjson.JSONEncodeError) as cm:
             orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
-        assert str(cm.exception) == "unsupported datatype in numpy array"
-        self.assertEqual(
-            orjson.dumps(
-                array, default=numpy_default, option=orjson.OPT_SERIALIZE_NUMPY
-            ),
-            orjson.dumps(array.tolist()),
-        )
+        assert "unsupported datatype in numpy array" in str(cm)
+        assert orjson.dumps(
+            array, default=numpy_default, option=orjson.OPT_SERIALIZE_NUMPY
+        ) == orjson.dumps(array.tolist())
 
     def test_numpy_array_d1(self):
         array = numpy.array([1])
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
     def test_numpy_array_d2(self):
         array = numpy.array([[1]])
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
     def test_numpy_array_d3(self):
         array = numpy.array([[[1]]])
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
     def test_numpy_array_d4(self):
         array = numpy.array([[[[1]]]])
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
     def test_numpy_array_4_stride(self):
         array = numpy.random.rand(4, 4, 4, 4)
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
     def test_numpy_array_dimension_zero(self):
         array = numpy.array(0)
         assert array.ndim == 0
-        with self.assertRaises(orjson.JSONEncodeError):
+        with pytest.raises(orjson.JSONEncodeError):
             orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
 
         array = numpy.empty((0, 4, 2))
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
         array = numpy.empty((4, 0, 2))
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
         array = numpy.empty((2, 4, 0))
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
     def test_numpy_array_dimension_max(self):
@@ -512,107 +518,114 @@ class NumpyTests(unittest.TestCase):
             1,
         )
         assert array.ndim == 32
-        self.assertEqual(
+        assert (
             orjson.loads(
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
                 )
-            ),
-            array.tolist(),
+            )
+            == array.tolist()
         )
 
     def test_numpy_scalar_int8(self):
-        self.assertEqual(
-            orjson.dumps(numpy.int8(0), option=orjson.OPT_SERIALIZE_NUMPY), b"0"
+        assert orjson.dumps(numpy.int8(0), option=orjson.OPT_SERIALIZE_NUMPY) == b"0"
+        assert (
+            orjson.dumps(numpy.int8(127), option=orjson.OPT_SERIALIZE_NUMPY) == b"127"
         )
-        self.assertEqual(
-            orjson.dumps(numpy.int8(127), option=orjson.OPT_SERIALIZE_NUMPY),
-            b"127",
+        assert (
+            orjson.dumps(numpy.int8(--128), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"-128"
         )
-        self.assertEqual(
-            orjson.dumps(numpy.int8(--128), option=orjson.OPT_SERIALIZE_NUMPY),
-            b"-128",
+
+    def test_numpy_scalar_int16(self):
+        assert orjson.dumps(numpy.int16(0), option=orjson.OPT_SERIALIZE_NUMPY) == b"0"
+        assert (
+            orjson.dumps(numpy.int16(32767), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"32767"
+        )
+        assert (
+            orjson.dumps(numpy.int16(-32768), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"-32768"
         )
 
     def test_numpy_scalar_int32(self):
-        self.assertEqual(
-            orjson.dumps(numpy.int32(1), option=orjson.OPT_SERIALIZE_NUMPY), b"1"
+        assert orjson.dumps(numpy.int32(1), option=orjson.OPT_SERIALIZE_NUMPY) == b"1"
+        assert (
+            orjson.dumps(numpy.int32(2147483647), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"2147483647"
         )
-        self.assertEqual(
-            orjson.dumps(numpy.int32(2147483647), option=orjson.OPT_SERIALIZE_NUMPY),
-            b"2147483647",
-        )
-        self.assertEqual(
-            orjson.dumps(numpy.int32(-2147483648), option=orjson.OPT_SERIALIZE_NUMPY),
-            b"-2147483648",
+        assert (
+            orjson.dumps(numpy.int32(-2147483648), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"-2147483648"
         )
 
     def test_numpy_scalar_int64(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.int64(-9223372036854775808), option=orjson.OPT_SERIALIZE_NUMPY
-            ),
-            b"-9223372036854775808",
+            )
+            == b"-9223372036854775808"
         )
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 numpy.int64(9223372036854775807), option=orjson.OPT_SERIALIZE_NUMPY
-            ),
-            b"9223372036854775807",
+            )
+            == b"9223372036854775807"
         )
 
     def test_numpy_scalar_uint8(self):
-        self.assertEqual(
-            orjson.dumps(numpy.uint8(0), option=orjson.OPT_SERIALIZE_NUMPY), b"0"
+        assert orjson.dumps(numpy.uint8(0), option=orjson.OPT_SERIALIZE_NUMPY) == b"0"
+        assert (
+            orjson.dumps(numpy.uint8(255), option=orjson.OPT_SERIALIZE_NUMPY) == b"255"
         )
-        self.assertEqual(
-            orjson.dumps(numpy.uint8(255), option=orjson.OPT_SERIALIZE_NUMPY),
-            b"255",
+
+    def test_numpy_scalar_uint16(self):
+        assert orjson.dumps(numpy.uint16(0), option=orjson.OPT_SERIALIZE_NUMPY) == b"0"
+        assert (
+            orjson.dumps(numpy.uint16(65535), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"65535"
         )
 
     def test_numpy_scalar_uint32(self):
-        self.assertEqual(
-            orjson.dumps(numpy.uint32(0), option=orjson.OPT_SERIALIZE_NUMPY), b"0"
-        )
-        self.assertEqual(
-            orjson.dumps(numpy.uint32(4294967295), option=orjson.OPT_SERIALIZE_NUMPY),
-            b"4294967295",
+        assert orjson.dumps(numpy.uint32(0), option=orjson.OPT_SERIALIZE_NUMPY) == b"0"
+        assert (
+            orjson.dumps(numpy.uint32(4294967295), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"4294967295"
         )
 
     def test_numpy_scalar_uint64(self):
-        self.assertEqual(
-            orjson.dumps(numpy.uint64(0), option=orjson.OPT_SERIALIZE_NUMPY), b"0"
-        )
-        self.assertEqual(
+        assert orjson.dumps(numpy.uint64(0), option=orjson.OPT_SERIALIZE_NUMPY) == b"0"
+        assert (
             orjson.dumps(
                 numpy.uint64(18446744073709551615), option=orjson.OPT_SERIALIZE_NUMPY
-            ),
-            b"18446744073709551615",
+            )
+            == b"18446744073709551615"
         )
 
     def test_numpy_scalar_float32(self):
-        self.assertEqual(
-            orjson.dumps(numpy.float32(1.0), option=orjson.OPT_SERIALIZE_NUMPY), b"1.0"
+        assert (
+            orjson.dumps(numpy.float32(1.0), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"1.0"
         )
 
     def test_numpy_scalar_float64(self):
-        self.assertEqual(
-            orjson.dumps(numpy.float64(123.123), option=orjson.OPT_SERIALIZE_NUMPY),
-            b"123.123",
+        assert (
+            orjson.dumps(numpy.float64(123.123), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"123.123"
         )
 
     def test_numpy_bool(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 {"a": numpy.bool_(True), "b": numpy.bool_(False)},
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'{"a":true,"b":false}',
+            )
+            == b'{"a":true,"b":false}'
         )
 
     def test_numpy_datetime(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 {
                     "year": numpy.datetime64("2021"),
@@ -626,12 +639,12 @@ class NumpyTests(unittest.TestCase):
                     "nano": numpy.datetime64("2021-01-01T00:00:00.172576789"),
                 },
                 option=orjson.OPT_SERIALIZE_NUMPY,
-            ),
-            b'{"year":"2021-01-01T00:00:00","month":"2021-01-01T00:00:00","day":"2021-01-01T00:00:00","hour":"2021-01-01T00:00:00","minute":"2021-01-01T00:00:00","second":"2021-01-01T00:00:00","milli":"2021-01-01T00:00:00.172000","micro":"2021-01-01T00:00:00.172576","nano":"2021-01-01T00:00:00.172576"}',
+            )
+            == b'{"year":"2021-01-01T00:00:00","month":"2021-01-01T00:00:00","day":"2021-01-01T00:00:00","hour":"2021-01-01T00:00:00","minute":"2021-01-01T00:00:00","second":"2021-01-01T00:00:00","milli":"2021-01-01T00:00:00.172000","micro":"2021-01-01T00:00:00.172576","nano":"2021-01-01T00:00:00.172576"}'
         )
 
     def test_numpy_datetime_naive_utc(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 {
                     "year": numpy.datetime64("2021"),
@@ -645,12 +658,12 @@ class NumpyTests(unittest.TestCase):
                     "nano": numpy.datetime64("2021-01-01T00:00:00.172576789"),
                 },
                 option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
-            ),
-            b'{"year":"2021-01-01T00:00:00+00:00","month":"2021-01-01T00:00:00+00:00","day":"2021-01-01T00:00:00+00:00","hour":"2021-01-01T00:00:00+00:00","minute":"2021-01-01T00:00:00+00:00","second":"2021-01-01T00:00:00+00:00","milli":"2021-01-01T00:00:00.172000+00:00","micro":"2021-01-01T00:00:00.172576+00:00","nano":"2021-01-01T00:00:00.172576+00:00"}',
+            )
+            == b'{"year":"2021-01-01T00:00:00+00:00","month":"2021-01-01T00:00:00+00:00","day":"2021-01-01T00:00:00+00:00","hour":"2021-01-01T00:00:00+00:00","minute":"2021-01-01T00:00:00+00:00","second":"2021-01-01T00:00:00+00:00","milli":"2021-01-01T00:00:00.172000+00:00","micro":"2021-01-01T00:00:00.172576+00:00","nano":"2021-01-01T00:00:00.172576+00:00"}'
         )
 
     def test_numpy_datetime_naive_utc_utc_z(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 {
                     "year": numpy.datetime64("2021"),
@@ -666,12 +679,12 @@ class NumpyTests(unittest.TestCase):
                 option=orjson.OPT_SERIALIZE_NUMPY
                 | orjson.OPT_NAIVE_UTC
                 | orjson.OPT_UTC_Z,
-            ),
-            b'{"year":"2021-01-01T00:00:00Z","month":"2021-01-01T00:00:00Z","day":"2021-01-01T00:00:00Z","hour":"2021-01-01T00:00:00Z","minute":"2021-01-01T00:00:00Z","second":"2021-01-01T00:00:00Z","milli":"2021-01-01T00:00:00.172000Z","micro":"2021-01-01T00:00:00.172576Z","nano":"2021-01-01T00:00:00.172576Z"}',
+            )
+            == b'{"year":"2021-01-01T00:00:00Z","month":"2021-01-01T00:00:00Z","day":"2021-01-01T00:00:00Z","hour":"2021-01-01T00:00:00Z","minute":"2021-01-01T00:00:00Z","second":"2021-01-01T00:00:00Z","milli":"2021-01-01T00:00:00.172000Z","micro":"2021-01-01T00:00:00.172576Z","nano":"2021-01-01T00:00:00.172576Z"}'
         )
 
     def test_numpy_datetime_omit_microseconds(self):
-        self.assertEqual(
+        assert (
             orjson.dumps(
                 {
                     "year": numpy.datetime64("2021"),
@@ -685,17 +698,77 @@ class NumpyTests(unittest.TestCase):
                     "nano": numpy.datetime64("2021-01-01T00:00:00.172576789"),
                 },
                 option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
-            ),
-            b'{"year":"2021-01-01T00:00:00","month":"2021-01-01T00:00:00","day":"2021-01-01T00:00:00","hour":"2021-01-01T00:00:00","minute":"2021-01-01T00:00:00","second":"2021-01-01T00:00:00","milli":"2021-01-01T00:00:00","micro":"2021-01-01T00:00:00","nano":"2021-01-01T00:00:00"}',
+            )
+            == b'{"year":"2021-01-01T00:00:00","month":"2021-01-01T00:00:00","day":"2021-01-01T00:00:00","hour":"2021-01-01T00:00:00","minute":"2021-01-01T00:00:00","second":"2021-01-01T00:00:00","milli":"2021-01-01T00:00:00","micro":"2021-01-01T00:00:00","nano":"2021-01-01T00:00:00"}'
         )
 
+    def test_numpy_datetime_nat(self):
+        with pytest.raises(orjson.JSONEncodeError):
+            orjson.dumps(numpy.datetime64("NaT"), option=orjson.OPT_SERIALIZE_NUMPY)
+        with pytest.raises(orjson.JSONEncodeError):
+            orjson.dumps([numpy.datetime64("NaT")], option=orjson.OPT_SERIALIZE_NUMPY)
+
     def test_numpy_repeated(self):
         data = numpy.array([[[1, 2], [3, 4], [5, 6], [7, 8]]], numpy.int64)
         for _ in range(0, 3):
-            self.assertEqual(
+            assert (
                 orjson.dumps(
                     data,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                ),
-                b"[[[1,2],[3,4],[5,6],[7,8]]]",
+                )
+                == b"[[[1,2],[3,4],[5,6],[7,8]]]"
             )
+
+
+@pytest.mark.skipif(numpy is None, reason="numpy is not installed")
+class TestNumpyEquivalence:
+    def _test(self, obj):
+        assert orjson.dumps(obj, option=orjson.OPT_SERIALIZE_NUMPY) == orjson.dumps(
+            obj.tolist()
+        )
+
+    def test_numpy_uint8(self):
+        self._test(numpy.array([0, 255], numpy.uint8))
+
+    def test_numpy_uint16(self):
+        self._test(numpy.array([0, 65535], numpy.uint16))
+
+    def test_numpy_uint32(self):
+        self._test(numpy.array([0, 4294967295], numpy.uint32))
+
+    def test_numpy_uint64(self):
+        self._test(numpy.array([0, 18446744073709551615], numpy.uint64))
+
+    def test_numpy_int8(self):
+        self._test(numpy.array([-128, 127], numpy.int8))
+
+    def test_numpy_int16(self):
+        self._test(numpy.array([-32768, 32767], numpy.int16))
+
+    def test_numpy_int32(self):
+        self._test(numpy.array([-2147483647, 2147483647], numpy.int32))
+
+    def test_numpy_int64(self):
+        self._test(
+            numpy.array([-9223372036854775807, 9223372036854775807], numpy.int64)
+        )
+
+    @pytest.mark.skip(reason="tolist() conversion results in 3.4028234663852886e38")
+    def test_numpy_float32(self):
+        self._test(
+            numpy.array(
+                [
+                    -340282346638528859811704183484516925440.0000000000000000,
+                    340282346638528859811704183484516925440.0000000000000000,
+                ],
+                numpy.float32,
+            )
+        )
+        self._test(numpy.array([-3.4028235e38, 3.4028235e38], numpy.float32))
+
+    def test_numpy_float64(self):
+        self._test(
+            numpy.array(
+                [-1.7976931348623157e308, 1.7976931348623157e308], numpy.float64
+            )
+        )
